import numpy as np
from scipy.spatial import cKDTree
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA

def smooth_pcl(points, sigma=3):
# def smooth_pcl(points, sigma=0.1):
    tree = cKDTree(points)

    # 高斯平滑参数
    radius = 3 * sigma
    # 平滑点云
    smoothed_points = np.zeros_like(points)
    for i, point in enumerate(points):
        neighbors_idx = tree.query_ball_point(point, r=radius)
        # print(len(neighbors_idx))
        distances = np.linalg.norm(points[neighbors_idx] - point, axis=1)
        weights = np.exp(-0.5 * (distances / sigma) ** 2)
        weights /= weights.sum()
        smoothed_points[i] = np.dot(weights, points[neighbors_idx])
    return smoothed_points

def align2ref(point_cloud):

    # Step 1: 使用 PCA 拟合平面
    pca = PCA(n_components=3)
    pca.fit(point_cloud)

    # 获取法向量（对应最小的特征值的特征向量）
    normal_vector = pca.components_[-1]  # 法向量为最小特征值对应的特征向量

    # Step 2: 计算旋转矩阵，将法向量对齐到 z 轴
    # 法向量 normal_vector 需要旋转到 [0, 0, 1]
    target_vector = np.array([0, 0, 1])

    # 叉积计算旋转轴
    axis = np.cross(normal_vector, target_vector)
    axis = axis / np.linalg.norm(axis)  # 单位化旋转轴

    # 点积计算旋转角度
    angle = np.arccos(np.dot(normal_vector, target_vector))

    # 使用罗德里格斯旋转公式计算旋转矩阵
    K = np.array([[0, -axis[2], axis[1]],
                [axis[2], 0, -axis[0]],
                [-axis[1], axis[0], 0]])

    rotation_matrix = np.eye(3) + np.sin(angle) * K + (1 - np.cos(angle)) * (K @ K)

    # Step 3: 应用旋转矩阵，校正点云
    aligned_point_cloud = point_cloud @ rotation_matrix.T

    # 可视化点云校平前后的结果
    # fig = plt.figure(figsize=(10, 5))

    # ax1 = fig.add_subplot(121, projection='3d')
    # ax1.scatter(point_cloud[:, 0], point_cloud[:, 1], point_cloud[:, 2], color='blue')
    # ax1.set_title('Original Point Cloud')
    # ax1.set_xlabel('X')
    # ax1.set_ylabel('Y')
    # ax1.set_zlabel('Z')

    # ax2 = fig.add_subplot(122, projection='3d')
    # ax2.scatter(aligned_point_cloud[:, 0], aligned_point_cloud[:, 1], aligned_point_cloud[:, 2], color='green')
    # ax2.set_title('Aligned Point Cloud')
    # ax2.set_xlabel('X')
    # ax2.set_ylabel('Y')
    # ax2.set_zlabel('Z')

    # plt.show()
    return aligned_point_cloud, rotation_matrix
