
import os
import matplotlib.pyplot as plt
from scipy.interpolate import griddata
from mpl_toolkits.mplot3d import Axes3D
import numpy as np
from scipy.sparse import lil_matrix
from scipy.sparse.linalg import spsolve

def plot_coords(world_points, camera_points, screen_points):

    # 选择每隔n个点进行采样
    n = 1000
    screen_points_sampled = screen_points[::n]
    camera_points_sampled = camera_points[::n]
    world_points_sampled = world_points[::n]
    # 创建一个3D坐标系
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    # 绘制点
    ax.scatter(screen_points_sampled[:, 0], screen_points_sampled[:, 1], screen_points_sampled[:, 2], c='blue', label='Screen')
    ax.scatter(camera_points_sampled[:, 0], camera_points_sampled[:, 1], camera_points_sampled[:, 2], c='yellow', label='Camera')
    ax.scatter(world_points_sampled[:, 0], world_points_sampled[:, 1], world_points_sampled[:, 2], c='magenta', label='Object')

    # 绘制线：相机点到被测物点的红色线
    for cam, obj in zip(camera_points_sampled, world_points_sampled):
        ax.plot([cam[0], obj[0]], [cam[1], obj[1]], [cam[2], obj[2]], 'r-')

    #绘制线：被测物点到屏幕点的绿色线
    for obj, scr in zip(world_points_sampled, screen_points_sampled):
        ax.plot([obj[0], scr[0]], [obj[1], scr[1]], [obj[2], scr[2]], 'g-')

    # 设置图例
    ax.legend()

    # 设置坐标轴标签
    ax.set_xlabel('X Axis')
    ax.set_ylabel('Y Axis')
    ax.set_zlabel('Z Axis')

    # 显示图形
    plt.show()